@@ -1,11 +1,11 @@
-use 5.010001;
+use 5.014;
 use strict;
 use warnings;
 
 package Test::FITesque::RDF;
 
 our $AUTHORITY = 'cpan:KJETILK';
-our $VERSION   = '0.008';
+our $VERSION   = '0.009';
 
 use Moo;
 use Attean::RDF;
@@ -15,11 +15,12 @@ use Test::FITesque::Test;
 use Types::Standard qw(InstanceOf);
 use Types::Namespace qw(Iri Namespace);
 use Types::Path::Tiny qw(Path);
-use Carp qw(croak);
+use Carp qw(carp croak);
 use Data::Dumper;
 use HTTP::Request;
 use HTTP::Response;
 use LWP::UserAgent;
+use Try::Tiny;
 
 has source => (
 					is      => 'ro',
@@ -62,7 +63,14 @@ sub transform_rdf {
   my $model = Attean->temporary_model;
 
   my $graph_id = iri('http://example.org/graph'); # TODO: Use a proper URI for graph
-  $model->add_iter($parser->parse_iter_from_io( $self->source->openr_utf8 )->as_quads($graph_id));
+
+  my $file_iter;
+  try {
+	 $file_iter = $parser->parse_iter_from_io( $self->source->openr_utf8 );
+  } catch {
+	 croak 'Failed to parse ' . $self->source . " due to $_";
+  };
+  $model->add_iter($file_iter->as_quads($graph_id));
 
   my $tests_uri_iter = $model->objects(undef, iri($ns->test->fixtures->as_string))->materialize; # TODO: Implement coercions in Attean
   if (scalar $tests_uri_iter->elements == 0) { # TODO: Better to check if there are fixture table entries that has no test
@@ -85,8 +93,10 @@ sub transform_rdf {
 		$params_base = URI::Namespace->new($params_base_term);
 		$ns->guess_and_add($params_base);
 	 }
-	 my $test_bgp = bgp(triplepattern($test_uri, iri($ns->test->handler->as_string), variable('handler')),
-							  triplepattern($test_uri, iri($ns->dc->identifier->as_string), variable('method')),
+	 my $test_bgp = bgp(triplepattern($test_uri, iri($ns->rdf->type->as_string), variable('handler_class')),
+							  triplepattern(variable('handler_class'), iri($ns->deps->iri('test-requirement')->as_string), variable('handler')), # Because Perl doesn't support dashes in method names
+							  triplepattern($test_uri, iri($ns->test->script->as_string), variable('method')),
+							  triplepattern($test_uri, iri($ns->dc->description->as_string), variable('description')),
 							  triplepattern($test_uri, iri($ns->test->params->as_string), variable('paramid')));
 
 	 my $algebra = Attean::Algebra::Query->new(children => [$test_bgp]); # TODO: generalize the next 4 lines in Attean
@@ -98,7 +108,7 @@ sub transform_rdf {
 		push(@instance, [$test->value('handler')->value]);
 		my $method = $test->value('method')->value;
 		my $params_iter = $model->get_quads($test->value('paramid')); # Get the parameters for each test
-		my $params;
+		my $params = {description => $test->value('description')->value}; # Description should always be present
 		while (my $param = $params_iter->next) {
 		  # First, see if there are HTTP request-responses that can be constructed
 		  my $req_head = $model->objects($param->subject, iri($ns->test->requests->as_string))->next;
@@ -164,6 +174,7 @@ sub transform_rdf {
 		}
 		push(@instance, [$method, $params])
 	 }
+	 carp 'Test was listed as ' . $test_uri->as_string . ' but not fully described' unless scalar @instance;
 	 push(@data, \@instance);
   }
   return \@data;
@@ -237,6 +248,39 @@ A L<IRI> to use in parsing the RDF fixture tables to resolve any relative URIs.
 
 =back
 
+=head2 REQUIRED RDF
+
+The following must exist in the test description (see below for an example and prefix expansions):
+
+=over
+
+=item C<< test:fixtures >>
+
+The object(s) of this predicate lists the test fixtures that will run
+for this test suite. May take an RDF List. Links to the test
+descriptions, which follow below.
+
+
+=item C<< test:script >>
+
+The object of this predicate contains the function name of the actual test script as a literal.
+
+=item C<< dc:description >>
+
+The object of this predicate provides a literal description of the test.
+
+=item C<< test:params >>
+
+The object of this predicate links to the paramaters, which may have many different shapes.
+
+=item C<< rdf:type >> aka C<<a>>
+
+The object of this predicate is the class of test, which again links
+contains the required C<< deps:test-requirement >> predicate, whose object
+contains the class name of the implementation of the tests.
+
+=back
+
 =head2 RDF EXAMPLE
 
 The below example starts with prefix declarations. Since this is a
@@ -246,12 +290,16 @@ using the C<test:fixtures> predicate will be used. Tests may be an RDF
 List, in which case, the tests will run in the specified sequence, if
 not, no sequence may be assumed.
 
-Then, two test fixtures are declared. The C<test:handler> predicate is
-used to identify the class containing implementations, while
-C<dc:identifier> is used to name the function within that class.
+Then, two test fixtures are declared. The RDF class of the test
+fixture is used to denote identify the Perl class containing the
+implementations, through the C<deps:test-requirement> predicate which is the
+concrete class name. The C<test:script> predicate is used to name the
+function within that class.
 
 The C<test:params> predicate is used to link the parameters that will
-be sent as a hashref into the function.
+be sent as a hashref into the function. The <dc:description> predicate
+is required to exist outside of the parameters, but will be included
+as a parameter as well.
 
 There are two different mechanisms for passing parameters to the test
 scripts, one is simply to pass arbitrary key-value pairs, the other is
@@ -271,27 +319,34 @@ resolution itself happens in L<URI::NamespaceMap>.
   @prefix deps: <http://ontologi.es/doap-deps#>.
   @prefix dc:   <http://purl.org/dc/terms/> .
   @prefix my:   <http://example.org/my-parameters#> .
-
+  @prefix rdfs: <http://www.w3.org/2000/01/rdf-schema#>.
 
   <#test-list> a test:FixtureTable ;
-    test:fixtures <#test1>, <#test2> .
+    test:fixtures ( <#test1> <#test2> ) .
 
-  <#test1> a test:Test ;
-    test:handler "Internal::Fixture::Simple"^^deps:CpanId ;
-    dc:identifier "string_found" ;
+  <#test1> a <http://example.org/SimpleTest> ;
     test:param_base <http://example.org/my-parameters#> ;
+    dc:description "Echo a string"@en ;
+    test:script "string_found" ;
     test:params [ my:all "counter-clockwise dahut" ] .
 
-  <#test2> a test:Test ;
-    test:handler "Internal::Fixture::Multi"^^deps:CpanId ;
-    dc:identifier "multiplication" ;
+  <#test2> a <http://example.org/MultiTest> ;
     test:param_base <http://example.org/my-parameters#> ;
+    test:script "multiplication" ;
+    dc:description "Multiply two numbers"@en ;
     test:params [
         my:factor1 6 ;
         my:factor2 7 ;
         my:product 42
     ] .
 
+  <http://example.org/SimpleTest> rdfs:subClassOf test:ScriptClass ;
+    deps:test-requirement "Internal::Fixture::Simple"^^deps:CpanId .
+
+  <http://example.org/MultiTest> rdfs:subClassOf test:ScriptClass ;
+    deps:test-requirement "Internal::Fixture::Multi"^^deps:CpanId .
+
+
 
 =head3 HTTP request-response lists
 
@@ -309,7 +364,7 @@ C<t/data/http-list.ttl> file for example.
 
 =head1 TODO
 
-Separate the implementation-specific details (such as C<test:handler>)
+Separate the implementation-specific details (such as C<deps:test-requirement>)
 from the actual fixture tables.
 
 =head1 BUGS
